#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Core;

namespace HeuristicLab.Encodings.SymbolicExpressionTreeEncoding {
  public interface ISymbolicExpressionTreeGrammar : IItem {
    IEnumerable<ISymbol> Symbols { get; }
    ISymbol StartSymbol { get; }
    void AddSymbol(ISymbol symbol);
    void RemoveSymbol(ISymbol symbol);

    bool ContainsSymbol(ISymbol symbol);
    void SetAllowedChild(ISymbol parent, ISymbol child, int argumentIndex);
    bool IsAllowedChild(ISymbol parent, ISymbol child, int argumentIndex);

    IEnumerable<ISymbol> GetAllowedSymbols(ISymbol parent, int argumentIndex);

    int GetMinExpressionLength(ISymbol start);
    int GetMaxExpressionLength(ISymbol start);
    int GetMinExpressionDepth(ISymbol start);

    int GetMinSubtreeCount(ISymbol symbol);
    void SetMinSubtreeCount(ISymbol symbol, int value);
    int GetMaxSubtreeCount(ISymbol symbol);
    void SetMaxSubtreeCount(ISymbol symbol, int value);
  }
}
